/**
 * @file
 *
 * @ingroup arm_beagle
 *
 * @brief Console configuration.
 */

/*
 * Copyright (c) 2012 Claas Ziemke. All rights reserved.
 *
 *  Claas Ziemke
 *  Kernerstrasse 11
 *  70182 Stuttgart
 *  Germany
 *  <claas.ziemke@gmx.net>
 *
 * The license and distribution terms for this file may be
 * found in the file LICENSE in this distribution or at
 * http://www.rtems.com/license/LICENSE.
 *
 * Modified by Ben Gras <beng@shrike-systems.com> to make
 * interrupt-driven uart i/o work for beagleboards; beaglebone support added.
 */

#include <libchip/serial.h>
#include <libchip/ns16550.h>

#include <bsp.h>
#include <bsp/irq.h>

#define CONSOLE_UART_THR (*(volatile unsigned int *)BSP_CONSOLE_UART_BASE)
#define CONSOLE_UART_RHR (*(volatile unsigned int *)BSP_CONSOLE_UART_BASE)
#define CONSOLE_UART_LSR (*(volatile unsigned int *)(BSP_CONSOLE_UART_BASE+0x14))
#define TX_FIFO_E (1<<5)
#define RX_FIFO_E (1<<0)

static uint8_t beagle_uart_get_register(uintptr_t addr, uint8_t i)
{
  uint8_t v;
  volatile uint32_t *reg = (volatile uint32_t *) addr;
  volatile uint32_t *reg_r = (volatile uint32_t *) addr + i;

  if(reg_r == BSP_CONSOLE_UART_BASE /* reading RHR */ ) {
    /* check there should be anything in the RHR before accessing it */
    if(!(CONSOLE_UART_LSR & 0x01)) {
      return 0;
    }
  }

  v = (uint8_t) *reg_r;

  return v;
}

static void beagle_uart_set_register(uintptr_t addr, uint8_t i, uint8_t val)
{
  volatile uint32_t *reg = (volatile uint32_t *) addr;

  reg [i] = val;
}

console_tbl Console_Configuration_Ports [] = {
    {
      .sDeviceName = "/dev/ttyS0",
      .deviceType = SERIAL_NS16550,
#if CONSOLE_POLLED
#warning polled
      .pDeviceFns = &ns16550_fns_polled,
#else
      .pDeviceFns = &ns16550_fns,
#endif
      .ulMargin = 16,
      .ulHysteresis = 8,
      .pDeviceParams = (void *) CONSOLE_BAUD,
      .ulCtrlPort1 = BSP_CONSOLE_UART_BASE,
      .ulDataPort = BSP_CONSOLE_UART_BASE,
      .ulIntVector = BSP_CONSOLE_UART_IRQ,
      .getRegister = beagle_uart_get_register,
      .setRegister = beagle_uart_set_register,
      .ulClock = UART_CLOCK,  /* 48MHz base clock */
    },
};

unsigned long Console_Configuration_Count = 1;

static void uart_write_polled( char c )
{
  while( ( CONSOLE_UART_LSR & TX_FIFO_E ) == 0 )
  	;
  CONSOLE_UART_THR = c;
  return;
}


static void _BSP_put_char( char c ) {
   uart_write_polled( c );
   if (c == '\n') {
       uart_write_polled('\r');
   }
}

BSP_output_char_function_type BSP_output_char = _BSP_put_char;
